// -----JS CODE-----
//@input Asset.ObjectPrefab objectPrefab

//@input float spawnFrequency{"widget":"slider","min":0.1, "max":5, "step":0.02}
//@input float spawnRandomizer{"widget":"slider","min":0, "max":0.5, "step":0.02}
//@input float spawnRange {"widget":"slider","min":0, "max":1, "step":0.1}
//@input float threshold {"widget":"slider","min":0, "max":0.5, "step":0.01}
//@input float fallingSpeedMin
//@input float fallingSpeedMax

//@input SceneObject mouthPositionObject
//@input Component.Camera camera

//@input Component.Text currentScoreNumber
//@input Component.Text missedScoreNumber

//@input float missedScoreMax
//@input Component.Text topScoreNumber

//game state related
//@input SceneObject StartScreen
//@input SceneObject ScoreScreen
//@input SceneObject EndScreen
//0--BeforeGameStart 1--DuringGame 2--GameEnded
var gameState = 0;
var currentScore = 0;
var nextSpawnTime =  (1/script.spawnFrequency); //reverse spawnFrequency so higher number would produce more frequent result, not necessary for our game but easier to understand.
var spawnRange = script.spawnRange;
var spawnTimer = 0;

//get screen position of this aka ObjectSpawner object
var screenTransform = script.getSceneObject().getComponent("Component.ScreenTransform");    
var myScreenPos = screenTransform.anchors.getCenter();

var missedScoreMax = script.missedScoreMax;
var missedScore = 0;
var topScore = 0;
var startgame = false;
var spawnedObjects = [];


//create reference to global persistent storage system
var store = global.persistentStorageSystem.store; 
//create a key to identify the variable to be saved
var scoreKey = "topScore";
//get from saved variable every time lens starts
topScore = store.getInt(scoreKey);
script.topScoreNumber.text = topScore.toString();

setState(0);

script.createEvent("UpdateEvent").bind(function(){
    
    if(startgame){
        getMouthPosition();    

        if(spawnTimer < nextSpawnTime){
            spawnTimer += getDeltaTime();
        }else{
            spawnObject();
            spawnTimer = 0;
            nextSpawnTime = 1.0/script.spawnFrequency + (Math.random() - 0.5) * script.spawnRandomizer;
        }
        
        if(missedScore >= missedScoreMax){
            onGameEnd();
        }
        
    }
});

global.behaviorSystem.addCustomTriggerResponse("START_GAME", onGameStart);

function onGameStart(){
   startgame = true;
   setState(1);
   currentScore = 0;
   missedScore = 0;
}

function onGameEnd(){
    startgame = false;
    clearSpawnedObjects();
    setState(2);
}

function setState(gameStateInt){
   switch(gameStateInt){
       case 0://before game start
           script.StartScreen.enabled = true;
           script.ScoreScreen.enabled = false;
           script.EndScreen.enabled = false;
       break;
       case 1://during game
           script.StartScreen.enabled = false;
           script.ScoreScreen.enabled = true;
           script.EndScreen.enabled = false;
       break;
       case 2://after game ended
           script.StartScreen.enabled = false;
           script.ScoreScreen.enabled = false;
           script.EndScreen.enabled = true;
       break;
   }
}

function spawnObject(){
    //creating a copy of the prefab   
    var newObj = script.objectPrefab.instantiate(script.getSceneObject().getParent());
    spawnedObjects.push(newObj);
    //randomize position with range
    var randomXpos = myScreenPos.x + Math.random()*spawnRange*2 - spawnRange;
    var newObjPosition = new vec2(randomXpos, myScreenPos.y);
    
    //set screen position of newObj aka ObjectPrefab object
    var objScreenTransform = newObj.getComponent("Component.ScreenTransform");
    objScreenTransform.anchors.setCenter(newObjPosition);
}

function clearSpawnedObjects(){
   for(i=0; i < spawnedObjects.length; i ++){
       spawnedObjects[i].destroy();
   }
   spawnedObjects = [];
}


function getFallingSpeed(){
    return Math.random() * (script.fallingSpeedMax - script.fallingSpeedMin) + script.fallingSpeedMin;
}

function getMouthPosition(){
    var mouthWorldPos = script.mouthPositionObject.getTransform().getWorldPosition();
    var mouthPos = script.camera.worldSpaceToScreenSpace(mouthWorldPos);
    mouthPos = new vec2(mouthPos.x*2-1, 1-mouthPos.y*2);
}

function getThreshold(){
    return script.threshold;
}


function OnHit(){
    if(startgame){
        currentScore ++;
        script.currentScoreNumber.text = currentScore.toString();
        if(currentScore > topScore){
            //update top score when current score exceeds top score
            topScore = currentScore;
            //store top score in persistent storage
            store.putInt(scoreKey, topScore);
            //update UI
            script.topScoreNumber.text = topScore.toString();
        }
    }
}


function OnMissed(){
    if(startgame){
        missedScore ++;
        script.missedScoreNumber.text = missedScore.toString();
    }
}

script.api.OnMissed = OnMissed;

script.api.OnHit = OnHit;
script.api.getMouthPosition = getMouthPosition;
script.api.getFallingSpeed = getFallingSpeed;
script.api.getThreshold = getThreshold;